/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: bruno.dillenseger@francetelecom.com
*
* Authors: Bruno Dillenseger
*/


import org.objectweb.mobilitools.smi.api.*;
import org.objectweb.mobilitools.smi.api.Agency;
import org.objectweb.mobilitools.smi.goodies.*;


/**
 * MobiliTools $Name:  $, $Id: EscapeAgent.java,v 1.1.1.1 2003/03/28 14:47:58 dillense Exp $
 * <P>
 * This agent example class shows how an agent may escape from an agency before it shuts down.
 * <BR>The easiest way to try this example is to run at least 2 GUI-enabled agencies, to
 * create at least one EscapeAgent instance in one of the agencies, and to shut it down.
 * <P>NB: THIS EXAMPLE REQUIRES THE MAFFINDER TO BE RUNNING BEFORE LAUNCHING AGENCIES AND AGENTS.
 */
public class EscapeAgent extends BasicMobileObject
{
	Name my_name;


	public EscapeAgent()
	{
	}


	public void afterBirth(AgentSystem agency, AgentInfo entry, Object argument)
		throws BadOperation
	{
		super.afterBirth(agency, entry, argument);
		my_name = entry.getName();
	}


	/**
	 * Get the list of agencies in current region, and escape to the first one in the list (if any).
	 */
	public synchronized void beforeShutdown()
	{
		try
		{
			Location[] locations = (new RegionManager(my_agency.getORB())).listAgencies(my_agency.getRegion(), null);
			if (locations.length > 0)
			{
				System.out.println(my_name + " is escaping to agency " + locations[0]);
				try
				{
					my_agency.moveLocalAgent(this, locations[0], "anyplace");
				}
				catch (BadOperation e)
				{
					System.err.println(e.getMessage());
				}
			}
			else
			{
				System.out.println("No agency available - " + my_name + " giving up.");
			}
		}
		catch (Exception e)
		{
			System.err.println("Can't get another agency - " + my_name + " giving up on exception:\n" + e.toString());
		}
	}


	public void beforeDeath()
	{
		System.out.println(my_name + " is dead.");
	}
}
