/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: bruno.dillenseger@francetelecom.com
*
* Authors: Bruno Dillenseger
*/


import org.objectweb.mobilitools.smi.api.*;
import org.objectweb.mobilitools.smi.goodies.BasicMobileObject;
import org.objectweb.mobilitools.smi.api.Agency;
import java.util.Properties;


/**
 * MobiliTools $Name:  $, $Id: PingPong.java,v 1.1.1.1 2003/03/28 14:47:58 dillense Exp $
 * <P>
 * This agent example class shows the visibility rules between classes of
 * agents loaded with different classloaders and, more important, with different codebases.
 * These rules typically depends on the class loader adapter (factory) used.
 * By default, SMI uses the SMIClassLoader which comes with the following principles:
 * <UL>
 * <LI>each agent has its own SMIClassLoader instance to get locally unresolved classes
 * <LI>however, classloaders with the same codebase share the same class objects
 * </UL>
 * As a result, agents with the same codebase may use/invoke each other's classes/instances,
 * while agents of different codebases will get an exception trying to do so.
 * <P>
 * The easiest way to use this example is to use GUI-enabled agencies. Create, suspend/resume
 * or move PingPong agents with a variety of codebases, to see how they can interact with each
 * other in an agency (have a look at the messages printed in the associated terminal).
 */
public class PingPong extends BasicMobileObject
{
	String my_name;


	public PingPong()
	{
	}


	/**
	 * This method is invoked on each other by PingPong agents.
	 */
	public String ping(PingPong agent)
	{
		return my_agency.getAgentInfo(agent).getName().toString() + " pings " + my_name;
	}


	////////////////////////////////
	// override BasicMobileObject //
	////////////////////////////////


	/**
	 * Set an appropriate property for the PingPong agent to be found by other PingPong agents.
	 */
	public void afterBirth(AgentSystem agency, AgentInfo info, Object arg)
	throws BadOperation
	{
		super.afterBirth(agency, info, arg);
		my_name = info.getName().toString();
		try
		{
			my_agency.setLocalAgentProperty(this, "ping-pong", "yes");
		}
		catch (BadOperation ex)
		{
			ex.printStackTrace();
		}
		beforeResume();
	}


	/**
	 * Lookup PingPong agents in current agency and try to invoke their <code>ping()</code> method
	 */
	public void beforeResume()
	{
		Properties prop = new Properties();
		prop.put("ping-pong", "yes");
		Name[] list = my_agency.listLocalAgents(my_agency.makeAgentProfile(prop));
		for (int i = 0 ; i < list.length ; ++i)
		{
			AgentInfo info = my_agency.getAgentInfo(list[i]);
			try
			{
				PingPong agent = (PingPong)my_agency.getAgentInfo(list[i]).getAgent();
				if (agent != this)
				{
					System.out.println(agent.ping(this));
				}
			}
			catch (ClassCastException ex)
			{
				System.err.println(my_name + " gets " + ex + " with agent " + info.getName());
			}
		}
	}


	/**
	 * Lookup PingPong agents in current agency and try to invoke their <code>ping()</code> method
	 */
	public void afterMove(AgentSystem agency, Location location, String place)
	throws BadOperation
	{
		super.afterMove(agency, location, place);
		beforeResume();
	}
}
