#!/bin/sh
CFGFILE=config.env
INCFILE=makefile.inc
DEFAULT_NSPORT=4807

# sets a parameter in both CFGFILE (sh form) and INCFILE (makefile form)
# usage:
# * setoption PARAMETER_NAME VALUE
#   -> directly sets the parameter
# * setoption PARAMETER_NAME DEFAULT VALUE PROMPT
#   -> prompts for another value before setting the parameter
setoption()
{
	VARNAME=$1
	DEFAULT="$2"
	S=""
	if test $# -eq 3
	then
		printf "%s %s: [%s] " "-" "$3" "$2"
		read S
	fi
	if test ! "$S"
	then
		S="$DEFAULT"
	fi
	printf "%s=%s\n" $VARNAME "$S" >> "$INCFILE"
	printf "%s=\"%s\"\n" $VARNAME "$S" >> "$CFGFILE"
	. "$CFGFILE"
}


echo "Makefile configurator for SMI"
echo > "$INCFILE"
if test "$1" = "-clean"
then
	rm -f "$CFGFILE"
	echo "Configuration cleaned."
elif test $# -eq 0
then
	if test -r "$CFGFILE"
	then
		. "$CFGFILE"
	fi
	echo "# configuration for SMI - generated by `basename $0`" > "$CFGFILE"
# set SMI root directory (SMIROOT)
	if test "$SMIROOT" = ""
	then
		SMIROOT=`pwd`
		SMIROOT=`dirname "$SMIROOT"`
		# transform "/cygdrive/<drive letter>/..." to "<drive letter>:/..." if using Cygwin
		case `uname -s` in
			CYGWIN* )
				if test "`echo $SMIROOT | cut -d / -f 2`" = "cygdrive"
				then
					SMIROOT=`echo $SMIROOT | cut -d / -f 3`:/`echo $SMIROOT | cut -d / -f 4-`
				fi
				;;
		esac
	fi
	setoption SMIROOT "$SMIROOT" "Enter SMI root directory"
# set preprocessor command
	case `uname -s` in
		SunOS )
			setoption CPP /usr/lib/cpp
			;;
		CYGWIN* | Linux | Darwin )
			setoption CPP /usr/bin/cpp
			;;
		* )
			setoption CPP cpp "Enter preprocessor command"
			;;
	esac
# set Java classpath separator character (CPS)
	case `uname -s` in
		CYGWIN* )
			setoption CPS ";"
			;;
		SunOS | Linux | Darwin )
			setoption CPS :
			;;
		* )
			setoption CPS ":" "Enter Java classpath separator"
	esac
# set JDK root directory (JDK) and JAVARUNTIME classes
	setoption JDK "$JDK" "Enter JDK root directory"
	case `uname -s` in
		Darwin )
			JAVARUNTIME=""
			for i in $JDK/../Classes/*.jar
			do
				JAVARUNTIME="$i$CPS$JAVARUNTIME"
			done
			setoption JAVARUNTIME "$JAVARUNTIME"
			;;
		SunOS | Linux | CYGWIN* )
			setoption JAVARUNTIME "$JDK/jre/lib/rt.jar"
			;;
		* )
			setoption JAVARUNTIME "$JDK/jre/lib/rt.jar" "Enter path to Java runtime classes"
	esac
# set Java compiler
	if test ! "$JCOMP"
	then
		JCOMP="$JDK/bin/javac"
	fi
	setoption JCOMP "$JCOMP" "Enter Java compiler command"
# choose ORB-specific definitions file
	if test ! "$ORBRULES"
	then
		ORBRULES="Java"
	fi
	echo "List of available ORB-specific rules files:"
	ls -1 ORBrules | grep -v CVS
	setoption ORBRULES "$ORBRULES" "Choose ORB-specific rules file"
# set server hostname for CORBA naming service
	if test ! "$SRVHOST"
	then
		SRVHOST=`uname -n`
	fi
	setoption SRVHOST "$SRVHOST" "Enter hostname of the server running CORBA naming service"
# set CORBA Naming service port number
	if test "$NSPORT" = ""
	then
		NSPORT=$DEFAULT_NSPORT
	fi
	setoption NSPORT "$NSPORT" "Enter port number for CORBA CosNaming service"
# set MAF region name
	setoption REGION "$REGION" "Enter MAF region name"
	cat makefile.base >> "$INCFILE"
else
	echo "usage: `basename $0` [-clean]"
fi
