/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi;


import org.objectweb.mobilitools.smi.api.*;
import java.util.Properties;


/**
 * MobiliTools $Name:  $, $Id: AgentEntry.java,v 1.1.1.1 2003/03/28 14:48:05 dillense Exp $
 * <P>
 * Data record holding information about an agent.
 */
public class AgentEntry implements AgentInfo
{
	MobileObject agent;
	Name name;
	String place;
	boolean running;
	String codebase;
	Properties properties;
	Object data;


	/**
		Creates a new data record describing an agent
		(with null as agency personality specific data)
		@param agent the Java object reference to the agent.
		@param name the MAF unique name of the agent.
		@param place the name of the place where the agent is residing.
		@param codebase the codebase for the agent's classes.
		@param properties a set of properties attached to the agent.
	*/
	public AgentEntry(
		MobileObject agent,
		Name name,
		String place,
		boolean running,
		String codebase,
		Properties properties)
	{
		this(agent, name, place, running, codebase, properties, null);
	}


	/**
		Creates a new data record describing an agent.
		@param agent the Java object reference to the agent.
		@param name the wrapper object of the agent's MAF unique name.
		@param place the name of the place where the agent is residing.
		@param codebase the codebase for the agent's classes.
		@param properties a set of properties attached to the agent.
		@param data opaque data that may be set by an agency personality.
	*/
	public AgentEntry(
		MobileObject agent,
		Name name,
		String place,
		boolean running,
		String codebase,
		Properties properties,
		Object data)
	{
		this.agent = agent;
		this.name = name;
		this.place = place;
		this.running = running;
		this.codebase = codebase;
		this.properties = properties;
		this.data = data;
	}


	/**
		@return the Java object reference to the agent.
	*/
	public MobileObject getAgent()
	{
		return agent;
	}


	/**
		@return the wrapper object of the agent's MAF name.
	*/
	public Name getName()
	{
		return name;
	}


	/**
		@return the name of the place where the agent is.
	*/
	public String getPlace()
	{
		return place;
	}


	/**
		@return true if the agent is currently active,
		false if it is suspended.
	*/
	public boolean isRunning()
	{
		return running;
	}


	/**
		Toggle the activity status between running and suspended.
		@return true if the new status is running, false if it is suspended.
	*/
	public boolean toggleRunning()
	{
		return running = ! running;
	}


	/**
		@return the codebase of the agent.
	*/
	public String getCodebase()
	{
		return codebase;
	}


	/**
		@return the set of properties attached to the agent.
	*/
	public Properties getProperties()
	{
		return properties;
	}


	/**
		@return the opaque data set by an agency personality.
	*/
	public Object getData()
	{
		return data;
	}
}
