/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi;


import org.omg.CfMAF.*;
import org.omg.CORBA.*;
import org.objectweb.mobilitools.smi.idl.*;


/**
 * MobiliTools $Name:  $, $Id: MAFFinder_impl.java,v 1.1.1.1 2003/03/28 14:48:05 dillense Exp $
 * <P>
 * Implementation of MAF's MAFFinder interface.
 * @see Finder
*/
public class MAFFinder_impl extends _MAFFinderComplementImplBase
	implements MAFFinderOperations, MAFFinderComplementOperations
{
	Finder myFinder;


	public MAFFinder_impl(Finder finder)
	{
		myFinder = finder;
	}


	///////////////////////////////////////////////////////
	// Implementation of MAF CORBA interface MAFFinder //
	///////////////////////////////////////////////////////


	/**
		Registers an agent if its name is unique, and if its identity is not empty.
	*/
	public void register_agent(
		Name agent_name,
		String agent_location,
		AgentProfile agent_profile)
		throws NameInvalid
	{
		myFinder.register(new FinderEntry(agent_name, agent_location, agent_profile));
	}


	/**
		Registers an agent system if its name is unique, and if its identity is not empty.
	*/
	public void register_agent_system(
		Name agent_system_name,
		String agent_system_location,
		AgentSystemInfo agent_system_info)
		throws NameInvalid
	{
		myFinder.register(new FinderEntry(agent_system_name, agent_system_location, agent_system_info));
	}


	/**
		Not implemented.
	*/
	public void register_place(
		String place_name,
		String place_location)
		throws NameInvalid, NO_IMPLEMENT
	{
		throw new NO_IMPLEMENT("MAFFinder operations for place management are not implemented.");
	}


	/**
		Lookup an agent from its name if the identity part is not empty,
		or lists all agents otherwise.
		@return location of agents
	*/
	public String[] lookup_agent(
		Name agent_name,
		AgentProfile agent_profile)
		throws EntryNotFound
	{
		return myFinder.lookup_locations(new org.objectweb.mobilitools.smi.api.Name(agent_name), null, agent_profile);
	}


	/**
		Lookup an agent system from its name if the identity part is not empty,
		or lists all agent systems otherwise.
		@return location of agent systems
	*/
	public String[] lookup_agent_system(
		Name agent_system_name,
		AgentSystemInfo agent_system_info)
		throws EntryNotFound
	{
		return myFinder.lookup_locations(new org.objectweb.mobilitools.smi.api.Name(agent_system_name), null, agent_system_info);
	}


	/**
		Not implemented.
	*/
	public String[] lookup_place(String place_name)
		throws EntryNotFound, NO_IMPLEMENT
	{
		throw new NO_IMPLEMENT("MAFFinder operations for place management are not implemented.");
	}


	/**
		Unregisters an agent.
	*/
	public void unregister_agent(Name agent_name)
		throws EntryNotFound
	{
		myFinder.unregister(new org.objectweb.mobilitools.smi.api.Name(agent_name));
	}


	/**
		Unregisters an agent system.
	*/
	public void unregister_agent_system(Name agent_system_name)
		throws EntryNotFound
	{
		myFinder.unregister(new org.objectweb.mobilitools.smi.api.Name(agent_system_name));
	}


	/**
		Not implemented.
	*/
	public void unregister_place(String place_name)
		throws EntryNotFound, NO_IMPLEMENT
	{
		throw new NO_IMPLEMENT("MAFFinder operations for place management are not implemented.");
	}


	///////////////////////////////////////////////////////////////
	// Implementation of SMI CORBA interface MAFFinderComplement //
	///////////////////////////////////////////////////////////////


	public Name[] lookup_agent_names(AgentProfile profile)
	{
		return myFinder.lookup_names(profile);
	}


	public AgentProfile get_agent_profile(Name name)
		throws AgentNotFound
	{
		return myFinder.getAgentProfile(new org.objectweb.mobilitools.smi.api.Name(name));
	}
}
