/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi;


import org.omg.CORBA.ORB;
import org.omg.CORBA.Any;
import org.omg.CfMAF.*;
import org.objectweb.mobilitools.smi.idl.*;
import org.objectweb.mobilitools.smi.api.Constants;
import java.util.Properties;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.net.URL;
import java.net.MalformedURLException;


abstract public class Misc
{
	static public final Name DUMMYNAME = newDummyName();
	static public final AgentProfile DUMMYAGENTPROFILE = newDummyAgentProfile();
	static public final AgentSystemInfo DUMMYAGENTSYSTEMINFO = newDummyAgentSystemInfo();


	static public Name newDummyName()
	{
		return new Name(new byte[0], new byte[0], (short)0);
	}


	static public AgentProfile newDummyAgentProfile()
	{
		return new AgentProfile(
			(short)0,
			(short)0,
			"",
			(short)0,
			(short)0,
			(short)0,
			new Any[0]);
	}


	static public AgentSystemInfo newDummyAgentSystemInfo()
	{
		return new AgentSystemInfo(
			newDummyName(),
			(short)0,
			new LanguageMap[0],
			"",
			(short)0,
			(short)0,
			new Any[0]);
	}


	static public Properties any2property(Any[] any)
	{
		Properties result = new Properties();
		if (any != null)
		{
			for (int i=0 ; i<any.length ; ++i)
			{
				SMIProperty prop = SMIPropertyHelper.extract(any[i]);
				result.setProperty(prop.name, prop.value);
			}
		}
		return result;
	}


	static public Any[] property2any(Properties properties, ORB orb)
	{
		Any[] result;
		if (properties != null)
		{
			result = new Any[properties.size()];
			Enumeration names = properties.propertyNames();
			for (int i=0 ; names.hasMoreElements() ; ++i)
			{
				String name = (String)names.nextElement();
				String value = properties.getProperty(name);
				if (value == null)
				{
					value = "";
				}
				result[i] = orb.create_any();
				SMIPropertyHelper.insert(result[i], new SMIProperty(name, value));
			}
		}
		else
		{
			result = new Any[0];
		}
		return result;
	}


	/**
		@return true if profile (i.e. AgentProfile or AgentSystemInfo) value
		matches mask profile, false otherwise.
		@param mask a AgentSystemInfo or AgentProfile instance, which may contain
		zero values (i.e. zero for integer fields, empty string for string fields).
		A null mask matches any profile, provided that the Profile type is the same
		(i.e. AgentProfile for both the mask and the value, or AgentSystemInfo for both).
		@param value an actual, complete AgentSystemInfo or AgentProfile instance,
		that has to be matched against the mask parameter. A null value matches with no mask
		but the null mask.
	*/
	static public boolean matchProfile(Object mask, Object value)
	{
		boolean result = false;
		if (mask == null)
		{
			result = true;
		}
		else if (value != null)
		{
			if (mask instanceof AgentProfile && value instanceof AgentProfile)
			{
				AgentProfile m = (AgentProfile)mask;
				AgentProfile v = (AgentProfile)value;
				result =
					(m.language_id == 0 || m.language_id == v.language_id) &&
					(m.agent_system_type == 0 || m.agent_system_type == v.agent_system_type) &&
					(m.major_version == 0 || m.major_version == v.major_version) &&
					(m.minor_version == 0 || m.minor_version == v.minor_version) &&
					(m.serialization == 0 || m.serialization == v.serialization) &&
					matchProperties(Misc.any2property(m.properties), Misc.any2property(v.properties));
			}
			else if (mask instanceof AgentSystemInfo && value instanceof AgentSystemInfo)
			{
				AgentSystemInfo m = (AgentSystemInfo)mask;
				AgentSystemInfo v = (AgentSystemInfo)value;

				result =
					(m.agent_system_type == 0 || m.agent_system_type == v.agent_system_type) &&
					(m.major_version == 0 || m.major_version == v.major_version) &&
					(m.minor_version == 0 || m.minor_version == v.minor_version) &&
					matchProperties(Misc.any2property(m.properties), Misc.any2property(v.properties));
			}
		}
		return result;
	}


	static public boolean matchProperties(Properties subset, Properties set)
	{
		Enumeration keys = subset.propertyNames();
		while (keys.hasMoreElements())
		{
			String key = (String)keys.nextElement();
			if (!subset.getProperty(key).equals(set.getProperty(key)))
			{
				return false;
			}
		}
		return true;
	}


	/**
	 * Invokes the ClassLoader factory (as set by the appropriate system
	 * property) to get a classloader for the given parameters. Note that
	 * parameters may have various interpretations, or may be ignored, depending
	 * on the actual class loader.
	 * @param parent the returned classloader's parent
	 * @param codebase the codebase (form depends on the actual classloader)
	 * @param profile the agent profile (see MAF spec.)
	 * @param provider the MAFAgentSystem implementation providing where to get
	 * classes from
	 * @return the appropriate classloader instance
	 * @see org.objectweb.mobilitools.smi.lib.SMIClassLoader
	 * @see org.objectweb.mobilitools.smi.lib.RMIClassLoaderAdapter
	 * @see org.objectweb.mobilitools.smi.lib.URLClassLoaderAdapter
	 * @see org.objectweb.mobilitools.smi.api.Constants
	 */
	static public ClassLoader getClassLoader(
		ClassLoader parent,
		String codebase,
		AgentProfile profile,
		MAFAgentSystem provider)
	{
		ClassLoader loader = null;
		String loader_name = System.getProperty(
			Constants.classLoaderProp,
			Constants.classLoaderDefault);
		try
		{
			loader = (ClassLoader)Class.forName(loader_name)
				.getMethod(
					Constants.classLoaderMethod,
					new Class[] {
						ClassLoader.class,
						String.class,
						AgentProfile.class,
						MAFAgentSystem.class })
				.invoke(
					null,
					new Object[] {
						parent,
						codebase,
						profile,
						provider});
		}
		catch (Throwable ex)
		{
			throw new Error(
				"Bad classloader factory or bad parameters: "
				+ loader_name
				+ "." + Constants.classLoaderMethod
				+ "("
				+ parent + ", "
				+ codebase + ", "
				+ profile + ", "
				+ provider
				+ ")\n" + ex);
		}
		return loader;
	}


	static public URL[] codebase2URLs(String codebase)
		throws MalformedURLException
	{
		StringTokenizer parser = new StringTokenizer(codebase, " ");
		URL[] urls = new URL[parser.countTokens()];
		int i=0;
		while (parser.hasMoreTokens())
		{
			urls[i++] = new URL(parser.nextToken());
		}
		return urls;
	}
}
