/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi;


import java.io.*;
import org.omg.CfMAF.MAFAgentSystem;
import org.omg.CfMAF.AgentProfile;


/**
 * MobiliTools $Name:  $, $Id: SMIObjectInputStream.java,v 1.1.1.1 2003/03/28 14:48:05 dillense Exp $
 * <P>
 * Responsible for deserializing (with a specific classloader) agents
 * received by an agency.
*/
public class SMIObjectInputStream extends ObjectInputStream
{
	ClassLoader my_loader;


	public SMIObjectInputStream(
		InputStream stream,
		ClassLoader loader)
		throws StreamCorruptedException, IOException
	{
		super(stream);
		my_loader = loader;
	}


	public SMIObjectInputStream(
		InputStream stream,
		String codebase,
		AgentProfile profile,
		MAFAgentSystem provider)
		throws StreamCorruptedException, IOException
	{
		super(stream);
		my_loader = Misc.getClassLoader(
			Thread.currentThread().getContextClassLoader(),
			codebase,
			profile,
			provider);
	}


	protected Class resolveClass (ObjectStreamClass classDesc)
		throws IOException, ClassNotFoundException
	{
		try
		{
			return super.resolveClass(classDesc);
		}
		catch (ClassNotFoundException e)
		{
			return my_loader.loadClass(classDesc.getName());
		}
	}
}
