/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi.api;


import java.util.Properties;


/**
 * MobiliTools $Name:  $, $Id: AgentInfo.java,v 1.1.1.1 2003/03/28 14:48:05 dillense Exp $
 * <P>
 * Interface for getting information about an agent.
*/
public interface AgentInfo
{
	/**
		@return the Java object reference to the agent.
	*/
	public MobileObject getAgent();

	/**
		@return the agent's name.
	*/
	public Name getName();

	/**
		@return the name of the place where the agent resides.
	*/
	public String getPlace();

	/**
		@return true if the agent is currently active,
		false if it is suspended.
	*/
	public boolean isRunning();

	/**
		Toggle the activity status between running and suspended.
		@return true if the agent is now running,
		false if it is suspended.
	*/
	public boolean toggleRunning();

	/**
		@return the codebase of the agent's classes.
	*/
	public String getCodebase();

	/**
		@return the set of properties attached to the agent.
	*/
	public Properties getProperties();

	/**
		@return the opaque data set by an agency personality.
	*/
	public Object getData();
}
