/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi.api;


import java.lang.Comparable;
import java.io.Serializable;


/**
 * MobiliTools $Name:  $, $Id: Name.java,v 1.1.1.1 2003/03/28 14:48:06 dillense Exp $
 * <P>
 * Wrapper class for MAF's Name class. Uniquely identies an agent or an agency.
*/
public class Name implements Comparable, Serializable
{
	byte[] my_authority;
	byte[] my_identity;
	short my_type;


	/**
		Duplicates a Name.
		@param name the name to duplicate
	*/
	public Name(Name name)
	{
		this(name.authority(), name.identity(), name.type());
	}


	/**
		Creates a new Name object from a MAF name (IDL-defined type).
		@param name the MAF representation of the name to duplicate
	*/
	public Name(org.omg.CfMAF.Name name)
	{
		this(name.authority, name.identity, name.agent_system_type);
	}


	/**
		Creates a new Name object from the given authority, identity
		and agent system type identifier.
		@param authority the authority of the agent or agency designated byt the new name
		@param identity the identity of the agent or agency designated by the new name
		@param type the agenct system type of the agent or agency designated by the new name
	*/
	public Name(byte[] authority, byte[] identity, short type)
	{
		my_authority = authority;
		my_identity = identity;
		my_type = type;
	}


	/**
		Creates a new Name object from the given authority and identity in
		string representation, and agent system type identifier.
		@param authority the authority of the agent or agency designated byt the new name
		@param identity the identity of the agent or agency designated by the new name
		@param type the agent system type of the agent or agency designated by the new name
	*/
	public Name(String authority, String identity, short type)
	{
		this(authority.getBytes(), identity.getBytes(), type);
	}


	/**
		Creates a new Name from a string representation of a MAF name.
		@see #getStringRepresentation()
	*/
	public Name(String str)
	{
		this(
			new String(str.getBytes(), 1, str.getBytes()[0]),
			new String(str.getBytes(), 2 + str.getBytes()[0], str.getBytes()[1 + str.getBytes()[0]]),
			(short)Integer.parseInt(new String(
				str.getBytes(),
				str.getBytes()[1+str.getBytes()[0]] + str.getBytes()[0] + 2,
				str.getBytes().length - (str.getBytes()[1+str.getBytes()[0]] + str.getBytes()[0] + 2))));
	}


	/**
		@return the MAF name (IDL-defined type).
	*/
	public org.omg.CfMAF.Name getmafname()
	{
		return new org.omg.CfMAF.Name(my_authority, my_identity, my_type);
	}


	/**
		@return a string representation of the agent idendity.
	*/
	public String identity()
	{
		return new String(my_identity);
	}


	/**
		@return a string representation of the agent's authority.
	*/
	public String authority()
	{
		return new String(my_authority);
	}


	/**
		@return the agent system type Id
	*/
	public short type()
	{
		return my_type;
	}


	/**
		@return a printable string (partial) representation of the name
		using format "<I>identity</I> <B>(</B><I>authority</I><B>)</B>".
	*/
	public String toString()
	{
		return identity() + " (" + authority() + ")";
	}


	/**
		@return a string-coded full representation of the MAF name (not printable).
	*/
	public String getStringRepresentation()
	{
		byte[] byteRep = new byte[1 + my_authority.length + 1 + my_identity.length];
		int i, j;
		byteRep[0] = (byte) my_authority.length;
		for (i=0, j=1 ; i<my_authority.length ; ++i, ++j)
		{
			byteRep[j] = my_authority[i];
		}
		byteRep[j++] = (byte) my_identity.length;
		for (i=0 ; i<my_identity.length ; ++i, ++j)
		{
			byteRep[j] = my_identity[i];
		}
		return new String(byteRep) + String.valueOf(my_type);
	}


	public boolean equals(Object other)
	{
		Name otherName = null;
		if (other instanceof Name)
		{
			otherName = (Name)other;
		}
		else if (other instanceof org.omg.CfMAF.Name)
		{
			otherName = new Name((org.omg.CfMAF.Name)other);
		}
		return
			otherName != null &&
			identity().equals(otherName.identity()) &&
			authority().equals(otherName.authority()) &&
			my_type == otherName.type();
	}


	public int hashCode()
	{
		return getStringRepresentation().hashCode();
	}


	public int compareTo(Object other)
	{
		return getStringRepresentation().compareTo(((Name)other).getStringRepresentation());
	}
}
