/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi.goodies;


import java.awt.*;
import org.omg.CORBA.*;
import java.awt.event.*;
import java.util.Observer;
import java.util.Observable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.io.*;
import org.objectweb.mobilitools.smi.*;
import org.objectweb.mobilitools.smi.api.*;
import org.objectweb.mobilitools.util.gui.*;
import org.omg.CfMAF.*;


/**
 * MobiliTools $Name:  $, $Id: AgencyGUI.java,v 1.1.1.1 2003/03/28 14:48:06 dillense Exp $
 * <P>
 * GUI for SMI agency. May be run as a standalone programme which launches a new agency
 * and its GUI, or may be used to create a GUI for an existing agency.
*/
public class AgencyGUI
	extends Frame
	implements FormListener, Observer, ActionListener, ItemListener
{
	Button updateBtn, createBtn, terminateBtn, viewBtn, moveBtn, shutdownBtn, resumeBtn, suspendBtn;
	Panel bottomPnl;
	List namesLst;
	Checkbox manualChk;
	Agency agency;
	org.omg.CfMAF.Name[] names;
	static boolean standalone = false;


	/**
		Creates a new agency.
		@param args must specify in this order: the agency name, and the region name.
	*/
	static public void main(String[] args)
	{
		ORB orb = ORB.init(args, null);
		AgencyGUI.standalone = true;
		try
		{
			new AgencyGUI(new Agency(args[0], args[1], orb));
			orb.run();
		}
		catch (BadOperation e)
		{
			e.printStackTrace();
			System.exit(-1);
		}
	}


	/**
		Opens a GUI for the specified agency.
		@param the_agency the SMI agency to browse.
	*/
	public AgencyGUI(Agency the_agency)
	{
		agency = the_agency;
		agency.exitOnTerminate(true, -1);
		setLayout(new BorderLayout());
		Panel topPnl = new Panel();
		topPnl.setLayout(new FlowLayout());
		topPnl.add(manualChk = new Checkbox("manual update", false));
		manualChk.addItemListener(this);
		topPnl.add(updateBtn = new Button("update"));
		updateBtn.setEnabled(false);
		updateBtn.addActionListener(this);
		topPnl.add(createBtn = new Button("create"));
		createBtn.addActionListener(this);
		topPnl.add(shutdownBtn = new Button("shutdown"));
		shutdownBtn.addActionListener(this);
		add("North", topPnl);
		add("Center", namesLst = new List());
		namesLst.addActionListener(this);
		namesLst.addItemListener(this);
		bottomPnl = new Panel();
		bottomPnl.add(viewBtn = new Button("view"));
		viewBtn.addActionListener(this);
		bottomPnl.add(suspendBtn = new Button("suspend"));
		suspendBtn.addActionListener(this);
		bottomPnl.add(resumeBtn = new Button("resume"));
		resumeBtn.addActionListener(this);
		bottomPnl.add(moveBtn = new Button("move"));
		moveBtn.addActionListener(this);
		bottomPnl.add(terminateBtn = new Button("terminate"));
		terminateBtn.addActionListener(this);
		add("South", bottomPnl);
		setTitle(
			"SMI agency " +	agency.getName() +
			", region " + agency.getRegion());
		updateNames();
		the_agency.addObserver(this);
		addWindowListener(new OnWindowClosing());
		pack();
		show();
	}


	/**
		Updates the list of hosted agents in agency's GUI.
	*/
	synchronized void updateNames()
	{
		names = agency.listAgents((AgentProfile)null);
		if (namesLst.getItemCount() > 0)
		{
			namesLst.removeAll();
		}
		for (int i=0 ; i<names.length ; ++i)
		{
			namesLst.add(
				new String(names[i].identity) +
				" (" +
				new String(names[i].authority) +
				")");
		}
		bottomPnl.setEnabled(false);
	}


	/**
		Terminates every agent in the agency, and terminates the agency.
		If the agency GUI has been run as a standalone programme, the
		process exits and returns the specified code.
		@param exitCode the exit code returned to the environment by the exiting process.
	*/
	public void shutdown(int exitCode)
	{
		try
		{
			agency.terminate();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		dispose();
		if (standalone)
		{
			System.exit(exitCode);
		}
	}


	//////////////////////////////////////////////
	// implementation of interface FormListener //
	//////////////////////////////////////////////

	public void formResult(String[] result)
	{
		if (result != null)
		{
			// creation form
			if (result[result.length-1].equals("create"))
			{
				StringTokenizer parser = new StringTokenizer(result[4], " ");
				String[] args = new String[parser.countTokens()];
				for (int i=0 ; i<args.length ; ++i)
				{
					args[i] = parser.nextToken();
				}
				try
				{
					agency.createLocalAgent(
						result[0],
						new org.objectweb.mobilitools.smi.api.Name(
							System.getProperty("user.name"),
							result[1],
							agency.getAgentSystemInfo().agent_system_type),
						result[2],
						result[3],
						new Properties(),
						args);
				}
				catch (BadOperation e)
				{
					new AlertWindow(
						this,
						"Can't create agent",
						e.getMessage(),
						"OK");
				}
			}
			// migration form
			else if (result[result.length-1].equals("move"))
			{
				try
				{
					agency.moveLocalAgent(
						new org.objectweb.mobilitools.smi.api.Name(names[namesLst.getSelectedIndex()]),
						new Location(result[0], result[1]),
						result[2]);
				}
				catch (BadOperation e)
				{
					new AlertWindow(
						this,
						"Can't move agent " + namesLst.getSelectedItem(),
						e.getMessage() + "\n(error code " + String.valueOf(e.getReason()) + ")",
						"OK");
				}
			}
			// shutdown confirmation form
			else if (result[result.length-1].equals("shutdown"))
			{
				shutdown(0);
			}
		}
	}


	//////////////////////////////////////////
	// Implementation of interface Observer //
	//////////////////////////////////////////


	public void update(Observable o, java.lang.Object arg)
	{
		if (arg == Agency.NOTIFY_TERMINATE)
		{
			dispose();
		}
		else
		{
			updateNames();
		}
	}


	////////////////////////////////////////////////
	// Implementation of interface ActionListener //
	////////////////////////////////////////////////


	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == updateBtn)
		{
			updateNames();
		}
		else if (e.getSource() == createBtn)
		{
			String[] fields = {
				"class name",
				"agent name",
				"place name",
				"code base",
				"arguments" };
			String[] buttons = {
				"create",
				"cancel" };
			FormFrame form = new FormFrame(
				"Create an agent",
				"Enter the following parameters:",
				fields,
				buttons);
			form.run(this);
		}
		else if (e.getSource() == terminateBtn)
		{
			try
			{
				agency.terminateLocalAgent(new org.objectweb.mobilitools.smi.api.Name(names[namesLst.getSelectedIndex()]));
			}
			catch (BadOperation ex)
			{
				new AlertWindow(
					this,
					"Warning",
					ex.getMessage(),
					"OK");
			}
		}
		else if (e.getSource() == viewBtn || e.getSource() == namesLst)
		{
			AgentInfo info = agency.getAgentInfo(
				new org.objectweb.mobilitools.smi.api.Name(names[namesLst.getSelectedIndex()]));
			StringWriter strWriter = new StringWriter();
			info.getProperties().list(new PrintWriter(strWriter));
			String message =
				"Name: " + info.getName().identity() + "\n" +
				"Authority: " + info.getName().authority() + "\n" +
				"Codebase: " + info.getCodebase() + "\n" +
				"Place: " + info.getPlace() + "\n" +
				"Activity: " + (info.isRunning() ? " running\n" : " suspended\n") +
				strWriter.toString();
			AlertWindow window = new AlertWindow(
				this,
				"Information on agent " + namesLst.getSelectedItem(),
				message,
				"OK");
		}
		else if (e.getSource() == moveBtn)
		{
			String[] fields = { "region", "agency", "place" };
			String[] buttons = { "move", "stay" };
			FormFrame moveFrame = new FormFrame(
				"Move agent " + namesLst.getSelectedItem(),
				"destination for agent " + namesLst.getSelectedItem(),
				fields,
				buttons);
			moveFrame.run(this);
		}
		else if (e.getSource() == shutdownBtn)
		{
			String[] buttons = {
				"shutdown",
				"cancel" };
			FormFrame form = new FormFrame(
				"Shutdown agency " + agency.getName() + "?",
				"Please confirm killing the agency and hosted agents.",
				new String[0],
				buttons);
			form.run(this);
		}
		else if (e.getSource() == suspendBtn)
		{
			try
			{
				agency.suspendLocalAgent(new org.objectweb.mobilitools.smi.api.Name(names[namesLst.getSelectedIndex()]));
			}
			catch (BadOperation ex)
			{
				if (ex.getReason() == BadOperation.UNKNOWNAGENT)
				{
					updateNames();
				}
				else
				{
					new AlertWindow(
						this,
						"Warning",
						ex.getMessage(),
						"OK");
				}
			}
		}
		else if (e.getSource() == resumeBtn)
		{
			try
			{
				agency.resumeLocalAgent(new org.objectweb.mobilitools.smi.api.Name(names[namesLst.getSelectedIndex()]));
			}
			catch (BadOperation ex)
			{
				if (ex.getReason() == BadOperation.UNKNOWNAGENT)
				{
					updateNames();
				}
				else
				{
					new AlertWindow(
						this,
						"Warning",
						ex.getMessage(),
						"OK");
				}
			}
 		}
	}


	//////////////////////////////////////////////
	// Implementation of interface ItemListener //
	//////////////////////////////////////////////


	public void itemStateChanged(ItemEvent e)
	{
		switch (e.getStateChange())
		{
		case ItemEvent.SELECTED:
			if (e.getSource() == manualChk)
			{
				updateBtn.setEnabled(true);
				agency.deleteObserver(this);
			}
			else if (e.getSource() == namesLst)
			{
				bottomPnl.setEnabled(true);
			}
			break;
		case ItemEvent.DESELECTED:
			if (e.getSource() == manualChk)
			{
				updateBtn.setEnabled(false);
				updateNames();
				agency.addObserver(this);
			}
			else if (e.getSource() == namesLst)
			{
				bottomPnl.setEnabled(false);
			}
			break;
		}
	}


	/////////////////////////////////////////
	// refinement of WindowAdapter methods //
	/////////////////////////////////////////


	class OnWindowClosing extends WindowAdapter
	{
		public void windowClosing(WindowEvent e)
		{
			AgencyGUI.this.shutdown(0);
		}
	}
}
