/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi.goodies;


import org.objectweb.mobilitools.smi.api.*;
import org.objectweb.mobilitools.smi.api.Agency;
import org.omg.CORBA.Any;
import org.omg.CfMAF.*;


/**
 * MobiliTools $Name:  $, $Id: BasicMobileObject.java,v 1.1.1.1 2003/03/28 14:48:06 dillense Exp $
 * <P>
 * Implementation of interface MobileObject, with (almost) empty methods, provided
 * as a helper base class to implement mobile objects.
 * Note the introduction of protected attribute <code>my_agency</code>, updated
 * by implementation of methods <code>afterBirth()</code> and <code>afterMove()</code>.
*/

public class BasicMobileObject implements MobileObject
{
	/** reference to the hosting agent system */
	protected transient Agency my_agency = null;


	/**
		Sets attribute my_agency to current host agency.
	*/
	public void afterBirth(AgentSystem agency, AgentInfo entry, Object argument)
		throws BadOperation
	{
		my_agency = (Agency)agency;
	}


	/**
		Empty method.
	*/
	public void beforeMove(Location location, String place)
		throws BadOperation
	{
	}


	/**
		Sets attribute my_agency to new host agency.
	*/
	public void afterMove(AgentSystem agency, Location location, String place)
		throws BadOperation
	{
		my_agency = (Agency)agency;
	}


	/**
		Empty method.
		@see MobileObject
	*/
	public void afterMoveFailed(
		Location location,
		String place,
		int reason,
		String message)
	{
	}


	/**
		Empty method.
	*/
	public void beforeDeath()
	{
	}


	/**
		Empty method.
	*/
	public void beforeSuspend()
		throws BadOperation
	{
	}


	/**
		Empty method.
	*/
	public void beforeResume()
		throws BadOperation
	{
	}


	/**
		Empty method.
	*/
	public void beforeShutdown()
	{
	}
}
