/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi.lib;


import org.omg.CfMAF.AgentProfile;
import org.omg.CfMAF.MAFAgentSystem;
import java.net.MalformedURLException;
import java.net.URLClassLoader;
import java.net.URL;
import java.util.Hashtable;
import org.objectweb.mobilitools.smi.Misc;


/**
 * MobiliTools $Name:  $, $Id: URLClassLoaderAdapter.java,v 1.1.1.1 2003/03/28 14:48:06 dillense Exp $
 * <P>
 * Classloader factory providing URL classloaders.
 */
abstract public class URLClassLoaderAdapter
{
	static Hashtable loaders = new Hashtable();


	/**
	 * Creates and a new URLClassLoader instance for the given codebase,
	 * if and and only if it does not already exist. Otherwise, the existing
	 * instance for the given codebase is re-used.
	 * @param parent parent classloader
	 * @param codebase codebase represented by URL strings separated with blanks
	 * @param profile ignored
	 * @param provider ignored
	 * @return the URLClassLoader for the given codebase
	 */
	static public ClassLoader getClassLoader(
		ClassLoader parent,
		String codebase,
		AgentProfile profile,
		MAFAgentSystem provider)
	throws MalformedURLException
	{
		Object loader = loaders.get(codebase);
		if (loader == null)
		{
			loader = URLClassLoader.newInstance(
				Misc.codebase2URLs(codebase),
				parent);
			loaders.put(codebase, loader);
		}
		return (ClassLoader)loader;
	}
}
