/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.smi.lib.bytearray;


import org.objectweb.mobilitools.smi.lib.SMIClassLoader;
import java.io.*;
import java.net.*;


public class ByteArrayURLConnection extends URLConnection
{
	String codebase, name;
	SMIClassLoader loader;
	byte[] bytes;


	public ByteArrayURLConnection(URL url, SMIClassLoader classloader)
	{
		super(url);
		name = url.getRef();
		loader = classloader;
		connected = false;
	}


	public void connect()
	{
		if (! connected)
		{
			bytes = loader.getBytes(name);
			connected = true;
		}
	}


	public int getContentLength()
	{
		if (! connected)
		{
			connect();
		}
		return bytes.length;
	}


	public String getContentType()
	{
		String type = guessContentTypeFromName(name);
		if (type == null)
		{
			if (! connected)
			{
				connect();
			}
			try
			{
				type = guessContentTypeFromStream(new ByteArrayInputStream(bytes));
			}
			catch (IOException ex)
			{
			}
		}
		return type;
	}


	public Object getContent()
		throws IOException
	{
		if (! connected)
		{
			connect();
		}
		return bytes;
	}


	public InputStream getInputStream()
		throws IOException
	{
		if (! connected)
		{
			connect();
		}
		return new ByteArrayInputStream(bytes);
	}


	public OutputStream getOutputStream()
		throws IOException
	{
		throw new IOException("bytearray URLs cannot by written to");
	}


	public String toString()
	{
		return "ByteArrayURLConnection for " + loader + ", URL " + getURL();
	}
}
