/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.util.corba;


import org.omg.CORBA.*;
import java.io.*;
import java.applet.Applet;
import java.util.Properties;


/**
 * MobiliTools $Name:  $, $Id: CORBA.java,v 1.1.1.1 2003/03/28 14:48:06 dillense Exp $
 * <P>
 * Miscellaneous CORBA facilitators.
*/
abstract public class CORBA
{
	static ORB orbSingleton = null;


	static public void setOrb(ORB orb)
	{
		orbSingleton = orb;
	}


	static public ORB getOrb()
	{
		if (orbSingleton == null)
		{
			orbSingleton = ORB.init();
		}
		return orbSingleton;
	}


	static public ORB setOrb(String[] args, Properties props)
	{
		return orbSingleton = ORB.init(args, props);
	}


	static public ORB setOrb(Applet applet, Properties props)
	{
		return orbSingleton = ORB.init(applet, props);
	}


	static public boolean refToFile(ORB orb, org.omg.CORBA.Object ref, String filename)
	{
		try
		{
			String refStr = orb.object_to_string(ref);
			PrintWriter out = new PrintWriter(
				new FileOutputStream(filename));
			out.println(refStr);
			out.flush();
		}
		catch(IOException ex)
		{
			return false;
		}
		return true;
	}


	static public org.omg.CORBA.Object refFromFile(ORB orb, String filename)
	{
		String refStr = null;
		try
		{
			BufferedReader in =
				new BufferedReader(new FileReader(filename));
			refStr = in.readLine();
		}
		catch(IOException ex)
		{
			return null;
		}
		return orb.string_to_object(refStr);
	}


	static public Any object2any(ORB orb, java.lang.Object obj)
		throws BAD_PARAM
	{
		Any result = orb.create_any();
		if (obj instanceof Boolean)
		{
			result.insert_boolean(((Boolean)obj).booleanValue());
		}
		else if (obj instanceof Character)
		{
			result.insert_char(((Character)obj).charValue());
		}
		else if (obj instanceof Byte)
		{
			result.insert_octet(((Byte)obj).byteValue());
		}
		else if (obj instanceof String)
		{
			result.insert_string((String)obj);
		}
		else if (obj instanceof Short)
		{
			result.insert_short(((Short)obj).shortValue());
		}
		else if (obj instanceof Integer)
		{
			result.insert_long(((Integer)obj).intValue());
		}
		else if (obj instanceof Long)
		{
			result.insert_longlong(((Long)obj).longValue());
		}
		else if (obj instanceof Float)
		{
			result.insert_float(((Float)obj).floatValue());
		}
		else if (obj instanceof Double)
		{
			result.insert_double(((Double)obj).doubleValue());
		}
		else
		{
			throw new BAD_PARAM();
		}
		return result;
	}


	static public java.lang.Object any2object(Any any)
	{
		java.lang.Object result = null;
		switch(any.type().kind().value())
		{
		case TCKind._tk_boolean:
			result = new Boolean(any.extract_boolean());
			break;
		case TCKind._tk_char:
			result = new Character(any.extract_char());
			break;
		case TCKind._tk_octet:
			result = new Byte(any.extract_octet());
			break;
		case TCKind._tk_string:
			result = any.extract_string();
			break;
		case TCKind._tk_short:
			result = new Short(any.extract_short());
			break;
		case TCKind._tk_long:
			result = new Integer(any.extract_long());
			break;
		case TCKind._tk_longlong:
			result = new Long(any.extract_longlong());
			break;
		case TCKind._tk_float:
			result = new Float(any.extract_float());
			break;
		case TCKind._tk_double:
			result = new Float(any.extract_double());
			break;
		default:
			throw new BAD_PARAM();
		}
		return result;
	}
}
