/*
* MobiliTools: an implementation of the Object Management Group's
* Mobile Agent Facility specification.
* Copyright (C) 2003 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* MobiliTools $Name:  $
*
* Contact: mobilitools-smi@lists.debian-sf.objectweb.org
*
* Authors: Bruno Dillenseger
*/


package org.objectweb.mobilitools.util.gui;


import java.awt.*;
import java.awt.event.*;


public class FormFrame extends Frame implements ActionListener
{
	Button[] boutonsBtn;
	TextField[] fields;
	FormListener listener;  // object to call back for returning results


	public FormFrame(String titre, String comment, String[] champs, String[] boutons)
	{
		this(titre, comment, champs, null, null, boutons);
	}


	public FormFrame(String titre, String comment, String[] champs, String[] valeurs, String[] boutons)
	{
		this(titre, comment, champs, valeurs, null, boutons);
	}


	public FormFrame(String titre, String comment, String[] champs, boolean[] secret, String[] boutons)
	{
		this(titre, comment, champs, null, secret, boutons);
	}


	public FormFrame(String titre, String comment, String[] champs, String[] valeurs, boolean[] secret, String[] boutons)
	{
		super(titre);
		setLayout(new BorderLayout());

		// zone des explications
		Label texte = new Label(comment);
		add("North", texte);

		// zone des champs du formulaire
		Panel labelPanel = new Panel(), fieldPanel = new Panel();
		labelPanel.setLayout(new GridLayout(champs.length, 1));
		fieldPanel.setLayout(new GridLayout(champs.length, 1));
		fields = new TextField[champs.length];
		for (int i=0 ; i<champs.length ; ++i)
		{
			labelPanel.add(new Label(champs[i]));
			fieldPanel.add(fields[i] = new TextField());
			if (valeurs != null && valeurs[i] != null && valeurs[i].length() > 0)
			{
				fields[i].setText(valeurs[i]);
			}
			if (secret != null && secret[i])
			{
				fields[i].setEchoChar('*');
			}
		}
		add("West", labelPanel);
		add("Center", fieldPanel);

		// zone des boutons
		Panel boutonsPnl = new Panel();
		boutonsPnl.setLayout(new FlowLayout());
		boutonsBtn = new Button[boutons.length];
		for (int i=0 ; i<boutons.length ; ++i)
		{
			boutonsPnl.add(boutonsBtn[i] = new Button(boutons[i]));
			boutonsBtn[i].addActionListener(this);
		}
		add("South", boutonsPnl);
		addWindowListener(new OnWindowClosing());
		pack();
	}


	public void run(FormListener the_listener)
	{
		listener = the_listener;
		show();
	}


	////////////////////////////////////////////////
	// Implementation of interface ActionListener //
	////////////////////////////////////////////////


	public void actionPerformed(ActionEvent evt)
	{
		int i;
		String[] values = new String[1+fields.length];
		for (i=0 ; i<fields.length ; ++i)
		{
			values[i] = fields[i].getText();
		}
		values[i] = ((Button)evt.getSource()).getLabel();
		listener.formResult(values);
		dispose();
	}


	///////////////////////////////////////
	// inner WindowAdapter-derived class //
	///////////////////////////////////////


	class OnWindowClosing extends WindowAdapter
	{
		public void windowClosing(WindowEvent e)
		{
			if (listener != null)
			{
				FormFrame.this.listener.formResult(null);
			}
			FormFrame.this.dispose();
		}
	}
}
